@@ -555,8 +555,8 @@ class SliderMixin:
         on_change : callable
             An optional callback invoked when this slider's value changes.
 
-        args : tuple
-            An optional tuple of args to pass to the callback.
+        args : list or tuple
+            An optional list or tuple of args to pass to the callback.
 
         kwargs : dict
             An optional dict of kwargs to pass to the callback.
@@ -723,8 +723,7 @@ class SliderMixin:
             )
 
         # Simplify future logic by always making value a list
-        if single_value:
-            value = [value]
+        prepared_value: Sequence[SliderScalar] = [value] if single_value else value  # ty: ignore[invalid-assignment]
 
         def value_to_generic_type(v: Any) -> SliderProto.DataType.ValueType:
             if isinstance(v, Integral):
@@ -736,26 +735,32 @@ class SliderMixin:
         def all_same_type(items: Any) -> bool:
             return len(set(map(value_to_generic_type, items))) < 2
 
-        if not all_same_type(value):
+        if not all_same_type(prepared_value):
             raise StreamlitAPIException(
                 "Slider tuple/list components must be of the same type.\n"
-                f"But were: {list(map(type, value))}"
+                f"But were: {list(map(type, prepared_value))}"
             )
 
         data_type = (
-            SliderProto.INT if len(value) == 0 else value_to_generic_type(value[0])
+            SliderProto.INT
+            if len(prepared_value) == 0
+            else value_to_generic_type(prepared_value[0])
         )
 
-        datetime_min = time.min
-        datetime_max = time.max
+        datetime_min: datetime | time = time.min
+        datetime_max: datetime | time = time.max
         if data_type == SliderProto.TIME:
-            datetime_min = time.min.replace(tzinfo=value[0].tzinfo)
-            datetime_max = time.max.replace(tzinfo=value[0].tzinfo)
-        if data_type in (SliderProto.DATETIME, SliderProto.DATE):
-            datetime_min = value[0] - timedelta(days=14)
-            datetime_max = value[0] + timedelta(days=14)
+            prepared_value = cast("Sequence[time]", prepared_value)
 
-        defaults: Final = {
+            datetime_min = time.min.replace(tzinfo=prepared_value[0].tzinfo)
+            datetime_max = time.max.replace(tzinfo=prepared_value[0].tzinfo)
+        if data_type in (SliderProto.DATETIME, SliderProto.DATE):
+            prepared_value = cast("Sequence[datetime]", prepared_value)
+
+            datetime_min = prepared_value[0] - timedelta(days=14)
+            datetime_max = prepared_value[0] + timedelta(days=14)
+
+        defaults: Final[dict[SliderProto.DataType.ValueType, dict[str, Any]]] = {
             SliderProto.INT: {
                 "min_value": 0,
                 "max_value": 100,
@@ -846,20 +851,20 @@ class SliderMixin:
         # Ensure that min <= value(s) <= max, adjusting the bounds as necessary.
         min_value = min(min_value, max_value)
         max_value = max(min_value, max_value)
-        if len(value) == 1:
-            min_value = min(value[0], min_value)
-            max_value = max(value[0], max_value)
-        elif len(value) == 2:
-            start, end = value
-            if start > end:
+        if len(prepared_value) == 1:
+            min_value = min(prepared_value[0], min_value)
+            max_value = max(prepared_value[0], max_value)
+        elif len(prepared_value) == 2:
+            start, end = prepared_value
+            if start > end:  # type: ignore[operator]
                 # Swap start and end, since they seem reversed
                 start, end = end, start
-                value = start, end
+                prepared_value = start, end
             min_value = min(start, min_value)
             max_value = max(end, max_value)
         else:
             # Empty list, so let's just use the outer bounds
-            value = [min_value, max_value]
+            prepared_value = [min_value, max_value]
 
         # Bounds checks. JSNumber produces human-readable exceptions that
         # we simply re-package as StreamlitAPIExceptions.
@@ -881,12 +886,20 @@ class SliderMixin:
         orig_tz = None
         # Convert dates or times into datetimes
         if data_type == SliderProto.TIME:
-            value = list(map(_time_to_datetime, value))
+            prepared_value = cast("Sequence[time]", prepared_value)
+            min_value = cast("time", min_value)
+            max_value = cast("time", max_value)
+
+            prepared_value = list(map(_time_to_datetime, prepared_value))
             min_value = _time_to_datetime(min_value)
             max_value = _time_to_datetime(max_value)
 
         if data_type == SliderProto.DATE:
-            value = list(map(_date_to_datetime, value))
+            prepared_value = cast("Sequence[date]", prepared_value)
+            min_value = cast("date", min_value)
+            max_value = cast("date", max_value)
+
+            prepared_value = list(map(_date_to_datetime, prepared_value))
             min_value = _date_to_datetime(min_value)
             max_value = _date_to_datetime(max_value)
 
@@ -900,17 +913,25 @@ class SliderMixin:
 
         # Now, convert to microseconds (so we can serialize datetime to a long)
         if data_type in TIMELIKE_TYPES:
+            prepared_value = cast("Sequence[datetime]", prepared_value)
+            min_value = cast("datetime", min_value)
+            max_value = cast("datetime", max_value)
+            step = cast("timedelta", step)
+
             # Restore times/datetimes to original timezone (dates are always naive)
             orig_tz = (
-                value[0].tzinfo
+                prepared_value[0].tzinfo
                 if data_type in (SliderProto.TIME, SliderProto.DATETIME)
                 else None
             )
 
-            value = list(map(_datetime_to_micros, value))
+            prepared_value = list(map(_datetime_to_micros, prepared_value))
             min_value = _datetime_to_micros(min_value)
             max_value = _datetime_to_micros(max_value)
-            step = _delta_to_micros(cast("timedelta", step))
+            step = _delta_to_micros(step)
+
+        # At this point, prepared_value is expected to be a list of floats:
+        prepared_value = cast("list[float]", prepared_value)
 
         # It would be great if we could guess the number of decimal places from
         # the `step` argument, but this would only be meaningful if step were a
@@ -922,7 +943,7 @@ class SliderMixin:
         slider_proto.id = element_id
         slider_proto.label = label
         slider_proto.format = format
-        slider_proto.default[:] = value
+        slider_proto.default[:] = prepared_value
         slider_proto.min = min_value
         slider_proto.max = max_value
         slider_proto.step = cast("float", step)
@@ -937,7 +958,12 @@ class SliderMixin:
         if help is not None:
             slider_proto.help = dedent(help)
 
-        serde = SliderSerde(value, data_type, single_value, orig_tz)
+        serde = SliderSerde(
+            prepared_value,
+            data_type,
+            single_value,
+            orig_tz,
+        )
 
         widget_state = register_widget(
             slider_proto.id,
@@ -953,16 +979,16 @@ class SliderMixin:
         if widget_state.value_changed:
             # Min/Max bounds checks when the value is updated.
             serialized_values = serde.serialize(widget_state.value)
-            for value in serialized_values:
+            for serialized_value in serialized_values:
                 # Use the deserialized values for more readable error messages for dates/times
-                deserialized_value = serde.deserialize_single_value(value)
+                deserialized_value = serde.deserialize_single_value(serialized_value)
 
-                if value < slider_proto.min:
+                if serialized_value < slider_proto.min:
                     raise StreamlitValueBelowMinError(
                         value=deserialized_value,
                         min_value=serde.deserialize_single_value(slider_proto.min),
                     )
-                if value > slider_proto.max:
+                if serialized_value > slider_proto.max:
                     raise StreamlitValueAboveMaxError(
                         value=deserialized_value,
                         max_value=serde.deserialize_single_value(slider_proto.max),
