@@ -97,7 +97,11 @@ def login(provider: str | None = None) -> None:
     more information, see Example 4.
 
     .. Important::
-        - You must install ``Authlib>=1.3.2`` to use this command.
+        - You must install ``Authlib>=1.3.2`` to use this command. You can
+          install it as an extra with Streamlit:
+
+          >>> pip install streamlit[auth]
+
         - Your authentication configuration is dependent on your host location.
           When you deploy your app, remember to update your ``redirect_uri``
           within your app and your provider.
@@ -266,6 +270,7 @@ def login(provider: str | None = None) -> None:
     Your app code:
 
     >>> import streamlit as st
+    >>>
     >>> if st.button("Log in"):
     >>>     st.login("auth0")
     >>> if st.user.is_logged_in:
